@@ -4,7 +4,8 @@
  * Xnee enables recording and replaying of X protocol data           
  *                                                                   
  * Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 
- *               2005, 2006, 2007, 2009  Henrik Sandklef 
+ *               2005, 2006, 2007, 2009, 2010
+ *               Henrik Sandklef 
  *                                                                   
  * This program is free software; you can redistribute it and/or     
  * modify it under the terms of the GNU General Public License       
@@ -210,8 +211,17 @@ xnee_expression_handle_replay(xnee_data *xd,
   int dummy3;
 
   if (!strncmp("0",tmp,1))  /* EVENT */
-    {    
-      xindata->oldtime = xindata->newtime ;
+    { 
+      unsigned long tmp_time; 
+      int last_type;
+      int last_x;
+      int last_y;
+
+      tmp_time = xindata->newtime;
+      last_type=xindata->u.event.type;
+      last_x=xindata->u.event.x;
+      last_y=xindata->u.event.y;
+
       ret = sscanf(tmp, "%d,%d,%d,%d,%d,%d,%d,%lu",
 		   &xindata->type, 
 		   &xindata->u.event.type, 
@@ -226,16 +236,29 @@ xnee_expression_handle_replay(xnee_data *xd,
 	  (void)xnee_print_error("Error in file %s \n", xd->data_name);
 	  ret = 0 ;
 	}   
+      
+      /* If the recorded sessions file has been recorded
+	 with XI enabled, we get one extra Motion event
+	 which will ¤% up the calc sleep funs: 
+	 If so: discard it */
+      if ( (xindata->u.event.screen_nr >= 0 ) && 
+	   (xindata->u.event.screen_nr < 1000))
+	{
+	  xindata->oldtime = tmp_time ;
+	}
+      else
+	{
+	  xindata->newtime = tmp_time ;
+	}
+      
     }   
   else if (!strncmp("1",tmp,1))  /* REQUEST */
     {
       xindata->oldtime = xindata->newtime ;
       ret = sscanf(tmp, "%d,%d,%lu",
-		      &xindata->type, 
-		      &xindata->u.request.type,
-		   &dummy1,
-		   &dummy2,
-		      &xindata->newtime);
+		   &xindata->type, 
+		   &xindata->u.request.type,
+		   &xindata->newtime);
       if (ret < 3)
 	{
 	  (void)xnee_print_error("Error in file %s \n", xd->data_name);
@@ -267,6 +290,52 @@ xnee_expression_handle_replay(xnee_data *xd,
 	  (void)xnee_print_error("Error in file %s \n", xd->data_name);
 	  ret = 0;
 	} 
+    }
+  else if (!strncmp("6",tmp,1))    /* XInput Device (master) */
+    {
+      if ( ! xnee_is_forced_core_device_events(xd))
+	{
+	  xindata->oldtime = xindata->newtime ;
+	  ret = sscanf(tmp, "%d,%d,%d,%d,%d,%d,%d,%lu,%d,%s",
+		       &xindata->type, 
+		       &xindata->u.xievent.type, 
+		       &xindata->u.xievent.x,
+		       &xindata->u.xievent.y, 
+		       &xindata->u.xievent.button,
+		       &xindata->u.xievent.keycode, 
+		       &xindata->u.xievent.screen_nr,
+		       &xindata->newtime,
+		       &xindata->u.xievent.deviceid,
+		       xindata->u.xievent.name);
+	  if (ret < 10)
+	    {
+	      (void)xnee_print_error("Error in file %s \n", xd->data_name);
+	      ret = 0;
+	    } 
+	}
+    }
+  else if (!strncmp("7",tmp,1))  /* XInput Device (slave) */
+    {
+      if ( ! xnee_is_forced_core_device_events(xd))
+	{
+	  xindata->oldtime = xindata->newtime ;
+	  ret = sscanf(tmp, "%d,%d,%d,%d,%d,%d,%d,%lu,%d,%s",
+		       &xindata->type, 
+		       &xindata->u.xievent.type, 
+		       &xindata->u.xievent.x,
+		       &xindata->u.xievent.y, 
+		       &xindata->u.xievent.button,
+		       &xindata->u.xievent.keycode, 
+		       &xindata->u.xievent.screen_nr,
+		       &xindata->newtime,
+		       &xindata->u.xievent.deviceid,
+		       xindata->u.xievent.name);
+	  if (ret < 10)
+	    {
+	      (void)xnee_print_error("Error in file %s \n", xd->data_name);
+	      ret = 0;
+	    } 
+	}
     }  
   else 
     {
@@ -333,10 +402,6 @@ xnee_expression_handle_settings(xnee_data *xd, char *tmp, int synt_mode)
 static int
 xnee_expression_handle_comment(xnee_data *xd, char *tmp)
 {
-  int len ;
-
-  len = strlen(tmp);
-
   xnee_verbose ((xd, "handling comment: \"%s\"\n", tmp));
   if (!strncmp("#",tmp,1))  /* # META data */
     {
@@ -709,7 +774,7 @@ xnee_expression_handle_prim(xnee_data *xd, char *str, xnee_intercept_data * xind
 {
   int ret= 0 ;  
   char *prim_args;
-  char buf[32];
+  char buf[256];
   int   prim_len = 0 ; 
   xnee_script_s xss ;
   
