@@ -3,7 +3,8 @@
  *                                                                   
  * Xnee enables recording and replaying of X protocol data           
  *                                                                   
- *        Copyright (C) 1999, 2000, 2001, 2002, 2003 Henrik Sandklef     
+ *        Copyright (C) 1999, 2000, 2001, 2002, 2003,
+ *                      2010  Henrik Sandklef     
  *                                                                   
  * This program is free software; you can redistribute it and/or     
  * modify it under the terms of the GNU General Public License       
@@ -34,10 +35,12 @@
 #include "libxnee/xnee_fake.h"
 #include "libxnee/xnee_keysym.h"
 #include "libxnee/xnee_display.h"
+#include "libxnee/xnee_xinput.h"
 
 
 static int loop_nr = 0;
 
+static int xnee_xi_last_know_pointer_pos[]={0,0};
 
 int
 xnee_reset_fake( xnee_data *xd)
@@ -71,50 +74,41 @@ xnee_fake_sleep(unsigned long period)
 
 
 
-
-/**************************************************************
- *                                                            *
- * xnee_replay_event_handler                                  *
- *                                                            *
- *                                                            *
- **************************************************************/
-int 
-xnee_replay_event_handler( xnee_data* xd, 
-			   xnee_intercept_data* xindata, 
-			   long int last_elapsed)
+static unsigned long
+xnee_replay_event_handler_sleep_amt(xnee_data* xd, 
+				    xnee_intercept_data* xindata, 
+				    long int last_elapsed,
+				    int      only_update_time)
 {
-  
-  int           return_value=0;
   unsigned long last_diff = 0;
   unsigned long first_diff = 0 ; 
   unsigned long record_last_diff = 0 ; 
   unsigned long sleep_amt = 0; 
   unsigned long record_first_diff =0 ;
-  int screen;
-  int x ;
-  int y ; 
   int speed ; 
-
   Time saved_time = 0 ; /* used to restore time of last replayable event */
 
-  xnee_verbose((xd, "---  xnee_replay_event_handler fake=%d\n ", (int)xd->fake));
-  XTestGrabControl (xd->fake, True); 
-  xnee_verbose((xd, "---  xnee_replay_event_handler 0\n "));
-  XFlush(xd->fake);                 
-
-  xnee_verbose((xd, "---  xnee_replay_event_handler 1\n "));
 
   /* Get the recorded time difference. 
      Should be from previous read 
      X server time recorded in file*/
   saved_time = xindata->oldtime ;
-  
+
+  if ( only_update_time )
+    {
+      xindata->oldtime = saved_time; 
+      return 0;
+    }
+
+
+
   /* Synchronise the time */
   xnee_verbose((xd,"ev handler  new %lu   old %lu\n",xindata->newtime , xindata->oldtime ));
   record_last_diff = xnee_delta_time(xindata) ; 
 
   record_first_diff = ( xindata->newtime - xd->first_read_time ) ;
 
+
   /* get the actual difference from last read - 
      we may have had to wait/sleep */
   last_diff = xnee_get_elapsed_time(xd, XNEE_FROM_LAST_READ );
@@ -122,6 +116,8 @@ xnee_replay_event_handler( xnee_data* xd,
   /* get the actual elapsed time from the start of the read */
   first_diff = xnee_get_elapsed_time(xd, XNEE_FROM_FIRST_READ );
   
+
+
   if (loop_nr==0)
     {
       record_last_diff = 0 ;
@@ -174,82 +170,216 @@ xnee_replay_event_handler( xnee_data* xd,
 				     record_last_diff, 
 				     record_first_diff ) ; 
     }
-
-/*   printf ("xnee_calc_sleep_amount : %lu : (%lu) : %lu %lu ?? %d\n",     */
-/* 	  last_diff, first_diff,  */
-/* 	  record_last_diff, record_first_diff, */
-/* 	  sleep_amt);    */
-
-
-
-  xnee_verbose((xd, "---  xnee_replay_event_handler switch on %d  (assuming %d or %d )\n ", xindata->u.event.type, KeyPress, KeyRelease));
   
-  /* If we use the last args to the XTestFakexxx functions
-   * it is harder to synchronize .... 
-   * XNEE_FAKE_SLEEP is a macro for usleep 
-   */
-  switch (xindata->u.event.type)
-    {
-    case KeyPress:
-      xnee_inc_events_replayed(xd);      
-      xnee_fake_key_event (xd, xindata->u.event.keycode, XNEE_PRESS, sleep_amt );
-      break;
-    case KeyRelease:
-      xnee_inc_events_replayed(xd);      
-      xnee_fake_key_event (xd, xindata->u.event.keycode, XNEE_RELEASE, sleep_amt);
-      break;
-    case ButtonPress:
-      xnee_inc_events_replayed(xd);      
-      xnee_fake_button_event (xd, xindata->u.event.button, XNEE_PRESS, sleep_amt);
-      break;
-    case ButtonRelease:
-      xnee_inc_events_replayed(xd);      
-      xnee_fake_button_event (xd, xindata->u.event.button, XNEE_RELEASE, sleep_amt);
-      break;
-    case MotionNotify:
-      xnee_inc_events_replayed(xd);      
-      screen = xindata->u.event.screen_nr ; 
-      x      = (int) xindata->u.event.x ; 
-      y      = (int) xindata->u.event.y ; 
-      xnee_fake_motion_event (xd,
-			      screen,
-			      x, 
-			      y, 
-			      sleep_amt);
-      break;
+  /*
+    printf ("xnee_calc_sleep_amount : %lu : (%lu) : %lu :  %lu ===>  %d\n",     
+   	  last_diff, first_diff,  
+   	  record_last_diff, record_first_diff, 
+   	  sleep_amt);    
+  */  
+  
+  
 
-    default:                  /*  Do nothing  */
-      xnee_verbose((xd, " Did NOT replay %d  returning XNEE_NOT_REPLAYABLE \n", xindata->u.event.type));
-      /* restore time of last replayable event */
-      xindata->oldtime = saved_time;
-      return_value= XNEE_NOT_REPLAYABLE;
-      break;
+  return sleep_amt;
+}
+
+
+/**************************************************************
+ *                                                            *
+ * xnee_replay_event_handler                                  *
+ *                                                            *
+ *                                                            *
+ **************************************************************/
+int 
+xnee_replay_event_handler( xnee_data* xd, 
+			   xnee_intercept_data* xindata, 
+			   long int last_elapsed)
+{
+  int           return_value=0;
+  unsigned long sleep_amt = 0; 
+
+  int screen;
+  int x ;
+  int y ; 
+
+
+  xnee_verbose((xd, "---  xnee_replay_event_handler fake=%d\n ", (int)xd->fake));
+  XTestGrabControl (xd->fake, True); 
+  xnee_verbose((xd, "---  xnee_replay_event_handler 0\n "));
+  XFlush(xd->fake);                 
+
+  xnee_verbose((xd, "---  xnee_replay_event_handler 1\n "));
+
+  xnee_verbose((xd, "---  xnee_replay_event_handler switch on %d  (assuming %d or %d )\n ", 
+		xindata->u.event.type, 
+		KeyPress, 
+		KeyRelease));
+  
+  if ( xnee_is_forced_core_device_events(xd))
+   {
+      if ( xindata->type == XNEE_PROTO_XINPUT_EVENT_SLAVE )
+	{
+	  ;
+	}
+      else
+	{
+	  /* If we use the last args to the XTestFakexxx functions
+	   * it is harder to synchronize .... 
+	   * XNEE_FAKE_SLEEP is a macro for usleep 
+	   */
+	  switch (xindata->u.event.type)
+	    {
+	    case KeyPress:
+	      sleep_amt = xnee_replay_event_handler_sleep_amt(xd, 
+							      xindata, 
+							      last_elapsed,
+							      0);
+	      xnee_inc_events_replayed(xd);      
+	      xnee_fake_key_event (xd, xindata->u.event.keycode, XNEE_PRESS, sleep_amt );
+	      break;
+	    case KeyRelease:
+	      sleep_amt = xnee_replay_event_handler_sleep_amt(xd, 
+							      xindata, 
+							      last_elapsed,
+							      0);
+	      xnee_inc_events_replayed(xd);      
+	      xnee_fake_key_event (xd, xindata->u.event.keycode, XNEE_RELEASE, sleep_amt);
+	      break;
+	    case ButtonPress:
+	      sleep_amt = xnee_replay_event_handler_sleep_amt(xd, 
+							      xindata, 
+							      last_elapsed,
+							      0);
+	      xnee_inc_events_replayed(xd);      
+	      xnee_fake_button_event(xd, xindata->u.event.button, XNEE_PRESS, sleep_amt);
+	      break;
+	    case ButtonRelease:
+	      sleep_amt = xnee_replay_event_handler_sleep_amt(xd, 
+							      xindata, 
+							      last_elapsed,
+							      0);
+	      xnee_inc_events_replayed(xd);      
+	      xnee_fake_button_event(xd, xindata->u.event.button, XNEE_RELEASE, sleep_amt);
+	      break;
+	    case MotionNotify:
+	      xnee_inc_events_replayed(xd);      
+	      screen = xindata->u.event.screen_nr ; 
+	      x      = (int) xindata->u.event.x ; 
+	      y      = (int) xindata->u.event.y ; 
+
+	      if ( (screen >= 0 ) && (screen < 1000))
+		{
+		  /* xnee_set_verbose(xd); */
+		  sleep_amt = xnee_replay_event_handler_sleep_amt(xd, 
+								  xindata, 
+								  last_elapsed,
+								  0);
+		  
+		  /*xnee_unset_verbose(xd); */
+		  xnee_fake_motion_event (xd,
+					  screen,
+					  x, 
+					  y, 
+					  sleep_amt);
+		}
+	      else 
+		{
+		  ;
+		}
+	      break;
+	      
+	    default:                  /*  Do nothing  */
+	      xnee_verbose((xd, " Did NOT replay %d  returning XNEE_NOT_REPLAYABLE \n", xindata->u.event.type));
+	      /* restore time of last replayable event */
+	      xnee_replay_event_handler_sleep_amt(xd, 
+						  xindata, 
+						  last_elapsed,
+						  1);
+	      return_value= XNEE_NOT_REPLAYABLE;
+	      break;
+	    }
+	}
     }
-
+  else
+    {
+      if ( xindata->type == XNEE_PROTO_XINPUT_EVENT_SLAVE )
+	{
+	  sleep_amt = xnee_replay_event_handler_sleep_amt(xd, 
+							  xindata, 
+							  last_elapsed,
+							  0);
+	
+	  if ( ( xindata->u.xievent.type >= KeyPress ) 
+	       && (xindata->u.xievent.type <= MotionNotify) )
+	    {
+	      int screen = xindata->u.event.screen_nr ; 
+	      int x      = (int) xindata->u.xievent.x ; 
+	      int y      = (int) xindata->u.xievent.y ; 
+	      int devid  = (int) xindata->u.xievent.deviceid ; 
+	      
+	      if ( xindata->u.xievent.type == MotionNotify) 
+		{
+		  xnee_fake_xi_motion_event (xd,
+					     screen,
+					     x, 
+					     y, 
+					     sleep_amt,
+					     devid);
+		}
+	      else if (xindata->u.xievent.type == ButtonPress) 
+		{
+		  xnee_inc_events_replayed(xd);      
+		  xnee_fake_xi_button_event(xd, xindata->u.event.button, 
+					    XNEE_PRESS, sleep_amt, devid);
+		}
+	      else if (xindata->u.xievent.type == ButtonRelease) 
+		{
+		  xnee_inc_events_replayed(xd);      
+		  xnee_fake_xi_button_event(xd, xindata->u.event.button, 
+					    XNEE_RELEASE, sleep_amt, devid);
+		}
+	      else if (xindata->u.xievent.type == KeyPress) 
+		{
+		  xnee_inc_events_replayed(xd);      
+		  xnee_fake_xi_key_event (xd, xindata->u.xievent.keycode, XNEE_PRESS, sleep_amt, devid);
+		}
+	      else if (xindata->u.xievent.type == KeyRelease) 
+		{
+		  xnee_inc_events_replayed(xd);      
+		  xnee_fake_xi_key_event (xd, xindata->u.xievent.keycode, XNEE_RELEASE, sleep_amt, devid);
+		}
+	    }
+	}
+      else
+	{
+	  ;
+	}
+    }
+  
   if (return_value==0)
     {
       ;
     }
   xnee_verbose((xd, "<--- xnee_replay_event_handler returning after handling of %d \n", 
 		xindata->u.event.type ));
-
   
-/*   printf ("xnee_replay_event_handler: %d\n", return_value); fflush(stdout); */
+  
+
   return return_value ;
 }
-
-
+  
+  
 
 
 
 /**************************************************************
  *                                                            *
- * xnee_fake_key_event                                        *
+ * xnee_fake_key_event_impl                                   *
  *                                                            *
  *                                                            *
  **************************************************************/
 int
-xnee_fake_key_event  (xnee_data* xd, int keycode, Bool bo, int dtime)
+xnee_fake_key_event_impl  (xnee_data* xd, int keycode, Bool bo, int dtime, int deviceid)
 {
   int i=0;
   int size= xd->distr_list_size;
@@ -257,28 +387,55 @@ xnee_fake_key_event  (xnee_data* xd, int keycode, Bool bo, int dtime)
 
   if (!xnee_is_recorder (xd))
     {
-      xnee_fake_sleep (dtime);
-      xnee_verbose((xd, "XTestFakeKeyEvent (%d, %d, %d, %d ))\n",
-		    (int) xd->fake, 
-		    (int) keycode, 
-		    (int) bo, 
-		    (int) dtime));
-      XTestFakeKeyEvent (xd->fake, keycode, bo, CurrentTime);
-      XFlush(xd->fake);
-    }
+      if (xnee_is_swinput_playback(xd))
+	{
+	  fprintf (stdout, "fake swinput\n");
+	  fprintf (stderr, "fake swinput\n");
+	}
+      else if (deviceid != 0 )
+	{
+	  XDevice *xdevice;
+	 
+	  xnee_set_verbose(xd);
+	  xdevice = xnee_get_xinput_device(xd, deviceid);
 
-  for (i=0; i<size ; i++)
-    {
-      XTestGrabControl (xd->distr_list[i].dpy, True); 
-      xnee_verbose((xd, "XTestFakeKeyEvent (%d, %d, %d, %d )) **\n",
-		    (int) xd->distr_list[i].dpy, 
-		    (int) keycode, 
-		    (int) bo, 
-		    (int) dtime));
-      XTestFakeKeyEvent (xd->distr_list[i].dpy, keycode, bo, dtime);
-      XFlush (xd->distr_list[i].dpy);
+	  xnee_unset_verbose(xd);
+	  xnee_fake_sleep (dtime); 
+	  
+
+	  XTestFakeDeviceKeyEvent(xd->fake, 
+				  xdevice, 
+				  keycode, 
+				  bo, 
+				  NULL,
+				  0,
+				  1);
+
+	}
+      else
+	{
+	  xnee_fake_sleep (dtime);
+	  xnee_verbose((xd, "XTestFakeKeyEvent (%d, %d, %d, %d ))\n",
+			(int) xd->fake, 
+			(int) keycode, 
+			(int) bo, 
+			(int) dtime));
+	  XTestFakeKeyEvent (xd->fake, keycode, bo, CurrentTime);
+	  XFlush(xd->fake);
+      
+	  for (i=0; i<size ; i++)
+	    {
+	      XTestGrabControl (xd->distr_list[i].dpy, True); 
+	      xnee_verbose((xd, "XTestFakeKeyEvent (%d, %d, %d, %d )) **\n",
+			    (int) xd->distr_list[i].dpy, 
+			    (int) keycode, 
+			    (int) bo, 
+			    (int) dtime));
+	      XTestFakeKeyEvent (xd->distr_list[i].dpy, keycode, bo, dtime);
+	    }
+	  XFlush (xd->distr_list[i].dpy);
+	}
     }
-  
   xnee_verbose((xd,"\n\n\n"));
   return (XNEE_OK);
 }
@@ -287,7 +444,7 @@ xnee_fake_key_event  (xnee_data* xd, int keycode, Bool bo, int dtime)
 
 /**************************************************************
  *                                                            *
- * xnee_fake_key_event                                        *
+ * xnee_fake_key_mod_event                                        *
  *                                                            *
  *                                                            *
  **************************************************************/
@@ -297,7 +454,7 @@ xnee_fake_key_mod_event (xnee_data* xd, xnee_script_s *xss, Bool bo, int dtime)
   int i=0;
   int size= xd->distr_list_size;
   int mods=0;
-  
+
 
   if (!xnee_is_recorder (xd))
     {
@@ -305,6 +462,7 @@ xnee_fake_key_mod_event (xnee_data* xd, xnee_script_s *xss, Bool bo, int dtime)
       for (mods=0;(mods<8)&&(xss->kc.mod_keycodes[mods]!=0);mods++)
 	{
 	  xnee_fake_sleep (dtime);
+
 	  xnee_fake_key_event (xd,
 			       xss->kc.mod_keycodes[mods], 
 			       bo, 
@@ -355,33 +513,61 @@ xnee_fake_key_mod_event (xnee_data* xd, xnee_script_s *xss, Bool bo, int dtime)
  *                                                            *
  **************************************************************/
 int
-xnee_fake_button_event (xnee_data* xd, int button, Bool bo , int dtime)
+xnee_fake_button_event_impl (xnee_data* xd, 
+			     int button, 
+			     Bool bo , 
+			     int dtime,
+			     int deviceid)
 {
   int i=0;
   int size= xd->distr_list_size;
   
   if (!xnee_is_recorder (xd))
     {
-      xnee_fake_sleep (dtime); 
-      xnee_verbose((xd, "XTestFakeButtonEvent (%d, %d, %d, %d)) \n",
-		    (int) xd->fake, 
-		    (int) button, 
-		    (int) bo, 
-		    (int) dtime));
-      XTestFakeButtonEvent (xd->fake, button, bo, 0);
-      XFlush(xd->fake);
-    }  
+      if (xnee_is_swinput_playback(xd))
+	{
+	  fprintf (stdout, "fake swinput\n");
+	  fprintf (stderr, "fake swinput\n");
+	}
+      else if (deviceid != 0 )
+	{
+	  XDevice *xdevice;
+	 
+	  xdevice = xnee_get_xinput_device(xd, deviceid);
 
-  for (i=0; i<size ; i++)
-    {
-      XTestGrabControl (xd->distr_list[i].dpy, True); 
-      xnee_verbose((xd, "XTestFakeButtonEvent (%d, %d, %d, %d))  **\n",
-		   (int) xd->distr_list[i].dpy, 
-		   (int) button, 
-		   (int) bo, 
-		   (int) dtime));
-      XTestFakeButtonEvent (xd->distr_list[i].dpy, button, bo, dtime);
-      XFlush (xd->distr_list[i].dpy);
+	  xnee_fake_sleep (dtime); 
+	  XTestFakeDeviceButtonEvent(xd->fake, 
+				     xdevice, 
+				     button, 
+				     bo, 
+				     xnee_xi_last_know_pointer_pos,
+				     2,
+				     1);
+	}
+      else
+	{
+	  xnee_fake_sleep (dtime); 
+	  xnee_verbose((xd, "XTestFakeButtonEvent (%d, %d, %d, %d)) \n",
+			(int) xd->fake, 
+			(int) button, 
+			(int) bo, 
+			(int) dtime));
+	  XTestFakeButtonEvent (xd->fake, button, bo, 0);
+	}  
+      
+      for (i=0; i<size ; i++)
+	{
+	  XTestGrabControl (xd->distr_list[i].dpy, True); 
+	  xnee_verbose((xd, "XTestFakeButtonEvent (%d, %d, %d, %d))  **\n",
+			(int) xd->distr_list[i].dpy, 
+			(int) button, 
+			(int) bo, 
+			(int) dtime));
+	  XTestFakeButtonEvent (xd->distr_list[i].dpy, button, bo, dtime);
+	  XFlush (xd->distr_list[i].dpy);
+	}
+      XFlush(xd->fake);
+      
     }
   return (XNEE_OK);
 }
@@ -396,60 +582,115 @@ xnee_fake_button_event (xnee_data* xd, int button, Bool bo , int dtime)
  *                                                            *
  **************************************************************/
 int
-xnee_fake_motion_event (xnee_data* xd,
-			int screen, 
-			int x, 
-			int y, 
-			unsigned long dtime)
+xnee_fake_motion_event_impl (xnee_data* xd,
+			     int screen, 
+			     int x, 
+			     int y, 
+			     unsigned long dtime,
+			     int deviceid)
 {
   int i=0;
   int size= xd->distr_list_size;
   int new_x;
   int new_y;
 
-	  
   xnee_verbose((xd, "---> xnee_fake_motion_event\n"));
   xnee_verbose((xd, "---  delay = %d\n", (int)dtime));
+
+  
   if (!xnee_is_recorder (xd))
     {
-
       new_x = xnee_resolution_newx(xd,x) + xd->res_info.x_offset;
       new_y = xnee_resolution_newy(xd,y) + xd->res_info.y_offset;
+      
+      if (xnee_is_swinput_playback(xd))
+	{
+	  fprintf (stdout, "fake swinput\n");
+	  fprintf (stderr, "fake swinput\n");
+	}
+      else if (deviceid == 0 )
+	{
+	  xnee_fake_sleep (dtime);
+	  xnee_verbose((xd, "XTestFakeMotionEvent (%d, %d, %d, %d, %d))\n",
+			(int) xd->fake, 
+			(int) screen, 
+			(int) new_x,
+			(int) new_y,
+			0));
 
-      xnee_fake_sleep (dtime);
-      xnee_verbose((xd, "XTestFakeMotionEvent (%d, %d, %d, %d, %d))\n",
-		    (int) xd->fake, 
-		    (int) screen, 
-		    (int) new_x,
-		    (int) new_y,
-		    0));
-      XTestFakeMotionEvent(xd->fake, 
-			   screen, 
-			   new_x, 
-			   new_y, 
-			   0);
+	  XTestFakeMotionEvent(xd->fake, 
+			       screen, 
+			       new_x, 
+			       new_y, 
+			       0);
+	}
+      else
+	{
+	  int axes[2];
+	  axes[0] = new_x;
+	  axes[1] = new_y;
+	  XDevice *xdevice;
+	  
+	  xnee_verbose((xd, "---  xi find device\n"));
+	 
+	  xdevice = xnee_get_xinput_device(xd, deviceid);
+
+	  xnee_verbose((xd, "---  xi devices found? \n"));
+
+	  if (xdevice==NULL)
+	    {
+	      fprintf(stderr, 
+		      "Couldn't find device for device id: %d\n",
+		      deviceid);
+	      return XNEE_XINPUT_EXTENSION_FAILURE;
+	    }
+	  
+	  xnee_fake_sleep (dtime);
+
+	  xnee_verbose((xd, "XTestFakeDeviceMotionEvent (%d, %d, %d, {%d, %d}, %d, %d))\n",
+			(int) xd->fake, 
+			(int) xdevice,
+			(int) False,
+			(int) new_x,
+			(int) new_y,
+			2,0));
+
+	  xnee_xi_last_know_pointer_pos[0]=axes[0];
+	  xnee_xi_last_know_pointer_pos[1]=axes[1];
+	  XTestFakeDeviceMotionEvent(xd->fake, 
+				     xdevice,
+				     False,
+				     0,
+				     axes,
+				     2,
+				     1);
+	}
+      
+      for (i=0; i<size ; i++)
+	{
+	  /*       XTestGrabControl (xd->distr_list[i].dpy, True);  */
+	  
+	  xnee_verbose((xd, "XTestFakeMotionEvent (%d, %d, %d, %d, %d))  **\n",
+			(int) xd->distr_list[i].dpy, 
+			(int) 0, 
+			(int) x,
+			(int) y,
+			10));
+	  XTestFakeMotionEvent(xd->distr_list[i].dpy, 
+			       0, 
+			       x, 
+			       y, 
+			       CurrentTime);
+	  XFlush (xd->distr_list[i].dpy);
+	  
+	}
+      
+      
       XFlush(xd->fake);
     }
+
+  
   
-  for (i=0; i<size ; i++)
-    {
-
-/*       XTestGrabControl (xd->distr_list[i].dpy, True);  */
-
-      xnee_verbose((xd, "XTestFakeMotionEvent (%d, %d, %d, %d, %d))  **\n",
-		   (int) xd->distr_list[i].dpy, 
-		   (int) 0, 
-		   (int) x,
-		   (int) y,
-		    10));
-      XTestFakeMotionEvent(xd->distr_list[i].dpy, 
-			   0, 
-			   x, 
-			   y, 
-			   CurrentTime);
-      XFlush (xd->distr_list[i].dpy);
-      
-    }
   xnee_verbose((xd, " <------- xnee_fake_motion_event\n"));
   return (XNEE_OK);
 }
@@ -522,7 +763,9 @@ xnee_type_file(xnee_data *xd)
   xnee_verbose ((xd,"--- xnee_type_file\n"));
 
   if (!xnee_has_xtest_extension(xd))
-    exit(XNEE_NO_TEST_EXT);
+    {
+      exit(XNEE_NO_TEST_EXT);
+    }
   
   xnee_verbose ((xd,"--- xnee_type_file\n"));
 
