@@ -552,10 +552,20 @@ xnee_record_from_data_display(xnee_data *xd)
 	      ret_val = 1;
 	    }
 	}
+      /* N900 */
+      else if (strstr(xd->x_vendor_name, "Nokia") )
+	{
+	  if ( ( xd->x_version_major == 1 ) &&
+	       ( xd->x_version_minor >= 1 ) )
+	    {
+	      ret_val = 1;
+	    }
+	}
       
     }
 
-  return ret_val;
+  return 1 ;
+  //ret_val;
 }
 
 
@@ -591,3 +601,21 @@ xnee_get_display_for_recordcontext(xnee_data *xd)
     }
   return context_display;
 }
+
+
+
+int 
+xnee_check_true(char *expr)
+{
+   return  (int) ( (strncmp(expr,XNEE_TRUE_STRING,strlen(XNEE_TRUE_STRING))==0)
+                   ||
+                   (strncmp(expr,XNEE_1_STRING,strlen(XNEE_1_STRING))==0) );
+}
+
+int 
+xnee_check_false(char *expr)
+{
+   return  (int)( (strncmp(expr,XNEE_FALSE_STRING,strlen(XNEE_FALSE_STRING))==0)
+                  ||
+                  (strncmp(expr,XNEE_0_STRING,strlen(XNEE_0_STRING))==0) );
+}
